/**********************************************************************************************
--This software was developed for DoD Joint Spectrum Center.  No distribution of 
--this software or development of new software from this listing is permitted 
--without expressed, written permission of the Commander, Joint Spectum Center.
--  2014 Exelis Inc, Government Unlimited Rights (DFARS 252.227-7014)
**********************************************************************************************/

// *************************************
// C++ routines for this program are wrappers to
// call the OSGEO GeoTiff library routines to 
// process the metadata for a GeoTiff format map image file.
// *****************

#include "targetver.h"

#include <stdio.h>
#include <string.h>

#include "xtiffio.h"
#include "geo_normalize.h"
//#include "geotiff.h"
//#include "geovalues.h"
//#include "tiffio.h"

#define DllExport  __declspec( dllexport )

extern "C" {
	double  DllExport _stdcall GeoTiffHelperVersion()
	{
		return 1.0;
	}
}

//using namespace std;
//********************************************************
//* Process GeoTiff Metadata and                         *
//* return Proj4 Projection string                       *
//********************************************************

extern "C" { // 2011/9/30 causes this method to be called externally like a C routine
	
	int DllExport _stdcall GetGeoTiffProj4String(char* fname, char *proj4String, int maxLengthReturn)
	{
		const char *CSVFileOverride( const char * );
		const char *CSVDirName = NULL;

		try
		{
			TIFF 	*tif=(TIFF*)0;  /* TIFF-level descriptor */
			GTIF	*gtif=(GTIF*)0; /* GeoKey-level descriptor */
			int		norm_print_flag = 1, proj4_print_flag = 0;
			int		tfw_flag = 0, inv_flag = 0, dec_flag = 0;

			if( fname == NULL )
			{
			//	Usage();
				goto failure;
			}

			/*
			 * Open the file, read the GeoTIFF information, and print to stdout. 
			 */

			tif=XTIFFOpen(fname,"r");
			if (!tif) goto failure;
	
			fprintf(stderr, "%s\n", fname);

			gtif = GTIFNew(tif);
			if (!gtif)
			{
				fprintf(stderr,"failed in GTIFNew\n");
				goto failure;
			}

			GTIFDefn	defn;
        
			if( GTIFGetDefn( gtif, &defn ) )
			{
				//if (!&defn)
				{
					//printf( "\n" );
					//printf( "PROJ.4 Definition: %s\n", GTIFGetProj4Defn(&defn));
					char* proj4 = GTIFGetProj4Defn(&defn);
					int proj4len = strlen(proj4);
					fprintf(stderr, "%d\n", proj4len);
					if (proj4len > 0)
					{
						fprintf(stderr, "%s\n", proj4);

						strncpy(proj4String, proj4, proj4len);
						fprintf(stderr, "%s\n", proj4String);
					}
				}
			}
		  //Success:
			GTIFFree(gtif);
			XTIFFClose(tif);
			return 0; 
		
		  failure:
			fprintf(stderr,"failure in getGeoPrj4ProjectionString\n");
			if (tif) XTIFFClose(tif);
			if (gtif) GTIFFree(gtif);
			return 200; //"Error: Could not get Proj4 String";
		}
		catch (...)
		{
			fprintf(stderr,"failed in GTIFGetDefn with exception\n");
		}
		return 300; //"Error: Could not get Proj4 String";
	}
}


